
#include <iostream>
#include <cuda_runtime.h>
#include <cuda.h>

#include <sys/time.h>

#define LOOP_SIZE 39999999
#define LIST_NUM	100
#define GPU_LOOP	1

using namespace std;

#define TIMESTAMP(s); {\
	timeval tv;\
	char time_str[100];\
	gettimeofday(&tv,NULL);\
	strftime(time_str, 100, "%F %H:%M:%S", localtime(&tv.tv_sec));\
	cout << time_str << ":" << tv.tv_usec << "    KProcess-1: " << s << endl;\
	}

__global__ void Kernel_dataCount(long* d_list, int num, clock_t* time)
{
	clock_t start = clock();
	int index = blockIdx.x * blockDim.x + threadIdx.x;

	for(int j = 0; j < GPU_LOOP; j ++){
		for(long i = 0; i < LOOP_SIZE; i ++){
			d_list[index] = num;
		}
	}
	*time = clock() - start;
}

void dataCount(int num)
{
	// GPU clock
	clock_t* d_time;
	clock_t endTime;

	long *d_list;
	int size = LIST_NUM * sizeof(long);
	long *list = new long[LIST_NUM];
	for(int i = 0; i < LIST_NUM; i++){
		list[i] = -1;
	}

	TIMESTAMP("Start!");

	// allocate resource
	cudaMalloc((void**) &d_list, size);
	cudaMalloc((void**) &d_time, sizeof(clock_t));
	cudaMemcpy(d_list, list, size, cudaMemcpyHostToDevice);

	// set Grid/Block/Thread
	if(LIST_NUM > 256){
		cout << "Error num" << endl;
		exit(1);
	}
	dim3 dimBlock(LIST_NUM);
	dim3 dimGrid(1);

	// run Kernel function
	TIMESTAMP("Call Kernel Function!");
	Kernel_dataCount<<<dimGrid, dimBlock>>>(d_list, num, d_time);
	TIMESTAMP("Call Kernel Function done!");

	// block until the device has completed
	cudaThreadSynchronize();
	TIMESTAMP("Synchronize done!");

	// Read result from device memory
	cudaMemcpy(list, d_list, size, cudaMemcpyDeviceToHost);
	cudaMemcpy(&endTime, d_time, sizeof(clock_t), cudaMemcpyDeviceToHost);
	TIMESTAMP("cudaMemcpy done!");

	for(int i = 0; i < LIST_NUM; i++){
		if (list[i] != num){
			cout << "!!Error!! list[" << i << "]= " << list[i] << endl;
		}
	}

	// Free device memory
	cudaFree(d_list);
	TIMESTAMP("cudaFree done!");
}

int main(int argc, char *argv[])
{
	int num=atoi(argv[1]);

	TIMESTAMP("Program Start!");
	dataCount(num);
	TIMESTAMP("Program Exit!");

	return 0;
}
